@@ -4,10 +4,22 @@ import os
 from apscheduler.executors.pool import ThreadPoolExecutor
 from apscheduler.jobstores.redis import RedisJobStore
 from apscheduler.schedulers.background import BackgroundScheduler
+from apscheduler.jobstores.base import ConflictingIdError, JobLookupError
+from apscheduler.util import datetime_to_utc_timestamp
 from fastapi import FastAPI
 from fastapi_utils.tasks import repeat_every
 from apscheduler.util import obj_to_ref
 
+try:
+    import cPickle as pickle
+except ImportError:  # pragma: nocover
+    import pickle
+
+try:
+    from redis import Redis
+except ImportError:  # pragma: nocover
+    raise ImportError('RedisJobStore requires redis installed')
+
 from fastapi_crawler_scheduler.utils.exception import SchedulerError
 from fastapi_crawler_scheduler.service.dbRedisHelper import DbRedisHelper
 from fastapi_crawler_scheduler.service.baseScheduler import BaseScheduler
@@ -26,8 +38,8 @@ class TaskScheduler(object):
                  job_misfire_grace_time: int = 10,
                  redis_host: str = "127.0.0.1",
                  redis_port: int = 6379,
-                 username: str = None,
-                 password: str = None,
+                 redis_username: str = None,
+                 redis_password: str = None,
                  ):
         self.app = app
         self.ssl = ssl
@@ -35,14 +47,25 @@ class TaskScheduler(object):
         self.uuid_number = uuid_number
         self.redis_host = redis_host
         self.redis_port = redis_port
-        self.username = username
-        self.password = password
+        self.username = redis_username
+        self.password = redis_password
         self.jobs_key = self.project_name + ':apscheduler:jobs:' + str(os.getpid())
         self.run_times_key = self.project_name + ':apscheduler:run_times:' + str(os.getpid())
         self.thread_pool_size = thread_pool_size
         self.job_coalesce = job_coalesce
         self.job_max_instance = job_max_instance
         self.job_misfire_grace_time = job_misfire_grace_time
+
+        self.redis = Redis(
+            host=self.redis_host,
+            port=self.redis_port,
+            username=self.username,
+            password=self.password,
+            decode_responses=True,
+            ssl=ssl,
+        )
+        self.pickle_protocol = pickle.HIGHEST_PROTOCOL
+
         # 实现 scheduler 注册
         self.register_scheduler()
         self.register_redis_db = DbRedisHelper(
@@ -58,35 +81,63 @@ class TaskScheduler(object):
             redis_db=self.register_redis_db,
             scheduler=self.scheduler,
             uuid_number=self.uuid_number,
-            redis_job_store=self.redis_job_store,
-            misfire_grace_time=self.job_misfire_grace_time,
-            coalesce=self.job_coalesce,
-            max_instances=self.job_max_instance,
+            redis_job_store=self.redis_job_store
         )
         self.register_async_task()
 
-    def register_async_task(self):
+    def register_async_task(self) -> None:
         @repeat_every(seconds=5)
-        def check_process():
+        def check_process() -> None:
             self.register_base_scheduler.check_process()
 
         @repeat_every(seconds=8)
-        def check_scheduler_run():
+        def check_scheduler_run() -> None:
             self.register_base_scheduler.run()
 
         @repeat_every(seconds=10)
-        def check_redis_jobstores():
+        def check_redis_jobstores() -> None:
             self.register_base_scheduler.check_redis_jobstores()
 
         def scheduler_start():
             self.scheduler.start()
 
+        def scheduler_shutdown():
+            self.scheduler.shutdown()
+
         self.app.on_event("startup")(check_process)
         self.app.on_event("startup")(check_scheduler_run)
         self.app.on_event("startup")(check_redis_jobstores)
         self.app.on_event("startup")(scheduler_start)
+        self.app.on_event("shutdown")(scheduler_shutdown)
 
-    def register_scheduler(self):
+    def add_job(self, job):
+        if self.redis.hexists(self.jobs_key, job.id):
+            raise ConflictingIdError(job.id)
+        self.redis.hset(self.jobs_key, job.id, pickle.dumps(job.__getstate__(), self.pickle_protocol))
+        if job.next_run_time:
+            self.redis.zadd(self.run_times_key, {job.id: datetime_to_utc_timestamp(job.next_run_time)})
+
+    def update_job(self, job):
+        if not self.redis.hexists(self.jobs_key, job.id):
+            raise JobLookupError(job.id)
+        self.redis.hset(self.jobs_key, job.id, pickle.dumps(job.__getstate__(), self.pickle_protocol))
+        if job.next_run_time:
+            self.redis.zadd(self.run_times_key,
+                            {job.id: datetime_to_utc_timestamp(job.next_run_time)})
+        else:
+            self.redis.zrem(self.run_times_key, job.id)
+
+    def remove_job(self, job_id):
+        if not self.redis.hexists(self.jobs_key, job_id):
+            raise JobLookupError(job_id)
+        self.redis.hdel(self.jobs_key, job_id)
+        self.redis.zrem(self.run_times_key, job_id)
+
+    def remove_all_jobs(self):
+        self.redis.delete(self.jobs_key)
+        self.redis.delete(self.run_times_key)
+
+    def register_scheduler(self) -> None:
         redis_job_store = getattr(self.app, "redis_job_store", None)
         if redis_job_store is None:
             redis_job_store = RedisJobStore(
@@ -96,7 +147,13 @@ class TaskScheduler(object):
                 password=self.password,
                 jobs_key=self.jobs_key,
                 run_times_key=self.run_times_key,
+                ssl=self.ssl,
             )
+            redis_job_store.add_job = self.add_job
+            redis_job_store.update_job = self.update_job
+            redis_job_store.remove_job = self.remove_job
+            redis_job_store.remove_all_jobs = self.remove_all_jobs
+
         self.redis_job_store = redis_job_store
         setattr(self.app, "redis_job_store", self.redis_job_store)
         scheduler = getattr(self.app, "scheduler", None)
@@ -131,7 +188,7 @@ class TaskScheduler(object):
             job_id: str,
             crawler_info: Dict = None,
             trigger: str = "interval",
-    ):
+    ) -> None:
         redis_dict = dict()
         redis_dict['func'] = obj_to_ref(func)
         redis_dict['interval'] = interval
@@ -148,7 +205,7 @@ class TaskScheduler(object):
             job_id: str,
             crawler_info: Dict = None,
             trigger: str = "interval",
-    ):
+    ) -> None:
         redis_dict = dict()
         redis_dict['func'] = obj_to_ref(func)
         redis_dict['interval'] = interval
@@ -161,5 +218,5 @@ class TaskScheduler(object):
     def delete_task(
             self,
             job_id: str,
-    ):
+    ) -> None:
         self.register_base_scheduler.delete_task(job_id=job_id)
